# Created 22 06 2022
# Here we will write the run? files

import display 
import PaperSearch as ps

#input
keyword_input = display.input()

#paper search
search1 = ps.PaperSearch(str(keyword_input)) 
if search1.paper:
    print("bibcode:", search1.paper)
    paper_info = search1.returnPaper()
    print("Paper Returned:", paper_info)
    title = paper_info[0]
    authors= paper_info[1]
    abstract = paper_info[2]  
    urls = paper_info[3]  

else: 
    print("No papers found")
    title = 'no title found for ' + keyword_input
    authors= 'NA'
    abstract = 'NA'
    urls = 'NA'

#display output
# display.output(title,authors,abstract,urls) 



#### Created 22 06 2022
## This file should be where all ADS functions are stored
import ads
import json

class PaperSearch(object):
    """
    PaperSearch class encapsulates paper search queries and returns. 
    """

    def __init__(self, keyword):
        """__init__ a search for a paper

        Args:
            keyword (str): the searched for word or phrase.
            token (string): ads provided token for seraching

        """

        # saving input
        self.keyword = 'abs:'+keyword
        ads.config.token = 'otQVJT3lALRZhXC4T7sRFrIF3HTtMf4xOlQ3SDpc'
        #this a token from a fake ads account I made with a temporary email. Should need to refresh token, but if necessary... 
        #LOGIN -- 
        #email: beyebi6100@syswift.com
        #password: luv2code 
        self.keywordSeach(self.keyword)

    def keywordSeach(self, keyword):
        """
        keywordsearch

        Searches for the keyword in abstracts and returns the first paper that matched with the highest citation count
        
        Args:
            keyword (string): search term. The users desired inquiry

        Returns:
            paper (string): ads provided bibcode for the top result paper
        """
        self.keyword = keyword
        papers = list(ads.SearchQuery(q=keyword, sort="citation_count",  fl=['id', 'bibcode', 'title', 'citation_count', 'author', 'abstract', 'citation'])) #sort="citation_count"


        if len(papers) > 0:
            self.paper = papers[0].bibcode
        else:
            self.paper = None
        return self.paper

    def returnPaper(self):
        """
        returnPaper

        This function returns the search's paper title, author list, and abstract
        
        Args:
            keyword (string): search term. The users desired inquiry

        Returns:
            title (string): ads provided bibcode for the top result paper
            author (string): ads first author
            abstract (string): ads provided abstract
        """
        paper = list(ads.SearchQuery(bibcode=self.paper, fl=['title', 'first_author', 'abstract','links_data']))[0]
        urls = [] #takes the link_data that's given and spits out just the url
        for i in paper.links_data: 
            urls.append(json.loads(i)["url"]) 
        
        import requests

        return [paper.title, paper.first_author, paper.abstract,urls]

    def returnCitation(self, n=5):
        """
        return citation

        This function returns the first n papers that cite the initial paper, ordered by citation count.

        Args: 
            n (int): number of articles to return. Default is 5

        Returns:
            list (list): n nested lists of citation articles information
                list[n, 0] (string): nth article's title
                list[n, 1] (string): nth article's first author
        """
        paper = list(ads.SearchQuery(bibcode=self.paper, fl=['title', 'first_author', 'abstract', 'citation']))[0]

        if not paper.citation:
            print("No citations for this article")
            return [None]*n

        cite_bibcodes=paper.citation
        cite_articles=[list(ads.SearchQuery(bibcode=bib, fl=['title','first_author', 'citation_count']))[0] for bib in cite_bibcodes]
        cite_sorted=sorted(cite_articles, key=lambda x: x.citation_count, reverse=True)
        cite_cut=cite_sorted[:n]
        return [[article.title, article.first_author] for article in cite_cut]

    def returnReference(self, n=5):
        """
        returns references

        This function returns the first n papers that cite the initial paper, ordered by citation count.

        Args: 
            n (int): number of articles to return. Default is 5

        Returns:
            list (list): n nested lists of citation articles information
                list[n, 0] (string): nth article's title
                list[n, 1] (string): nth article's first author

        """
        paper = list(ads.SearchQuery(bibcode=self.paper, fl=['title', 'author', 'abstract', 'reference']))[0]

        if not paper.reference:
            print("No reference for this article")
            return [None]*n

        ref_bibcodes=paper.reference[:n]
        ref_articles=[list(ads.SearchQuery(bibcode=bib, fl=['title','first_author', 'citation_count']))[0] for bib in ref_bibcodes]
        ref_sorted=sorted(ref_articles, key=lambda x: x.citation_count, reverse=True)
        ref_cut=ref_sorted[:n]
        return [[article.title, article.first_author] for article in ref_cut]


# search = PaperSearch('supernova').returnPaper()

# print (search)


import tkinter as tk
from tkinter import ttk
from tkinter.font import BOLD
from tkinter.tix import DisplayStyle
import webbrowser
import requests 
import validators
from PIL import ImageTk, Image

#Diplsay Input
def input():
    """
    Interactive box for Input Values

    Returns:
        the values that the user typed in the box:
        token_input (string): the users ads token
        keyword_input (string): the keyword the user gave
    """


    # root window
    root = tk.Tk()
    root.resizable(False, False)
    root.title('Astro MVP')
    root.geometry("500x250")

    #Add header image!
    frame = tk.Frame(root, width=600, height=400)
    frame.pack()
    frame.place(anchor='center', relx=0.5, rely=0.5)
    img = ImageTk.PhotoImage(Image.open("images/astromvplogo.png"))
    label = tk.Label(root, image = img)
    label.pack()



    # store keyword address and pub_year
    keyword = tk.StringVar()
    ads = tk.StringVar()

    # submit frame
    submit_frame = ttk.Frame(root)
    submit_frame.pack(padx=10, pady=10, fill='x', expand=True,side=tk.BOTTOM)

    #little intro:
    introtext = 'Welcome to Astro MVP. We use the Astrophysics Data System to \nfind the most valuable astronomy paper related to your keyword.\n  '
    intro_label = ttk.Label(submit_frame, text=introtext,font = ('Helvetica', 14))
    intro_label.pack(fill='none',expand=False) 


    # keywords
    keyword_label = ttk.Label(submit_frame, text="Keyword:",font = ('Helvetica', 14))
    keyword_label.pack(fill='x', expand=True)

    keyword_entry = ttk.Entry(submit_frame, textvariable=keyword)
    keyword_entry.pack(fill='x', expand=True)
    keyword_entry.focus()


    #THIS IS WHERE YOU GRAB THE VAlUES SUMBMITTED IN THE TEXTBOX
    keyword_input = []
    def submit_clicked():
        keyword_input.append(keyword.get())
        root.destroy()
        return keyword_input



    submit_button = ttk.Button(submit_frame, text="Submit", command=submit_clicked)
    submit_button.pack(fill='x', expand=True, pady=10)

    root.mainloop()


#Test Input Display 
# input()

#OUTPUT DISPLAY

def output(title,authors,abstract,urls): 

    """
    Displays the output (title,author,abstract) from papersearch.py in box

    """

    root = tk.Tk()
    root.title('Astro MVP')
    #Add header image!
    frame = tk.Frame(root) 
    frame.pack()
    frame.place(anchor='n', relx=0.0, rely=0.0)
    img = ImageTk.PhotoImage(Image.open("images/astromvplogo.png"))
    label = tk.Label(root, image = img)
    label.pack()

    S = tk.Scrollbar(root)
    T = tk.Text(root, height=35, width=40,font = ('Helvetica', 18),bd=0)
    S.pack(side=tk.TOP)
    T.pack(side=tk.TOP,fill='both',expand=True) 
    S.config(command=T.yview)
    T.config(yscrollcommand=S.set)

  
    def uri_exists(url_value):
        for i in url_value: 
            valid=validators.url(i) 
            if valid==True:
                url_value = i 
            else:
                url_value = 'https://ui.adsabs.harvard.edu/'

        return url_value

    the_url = uri_exists(urls)


    # title_quote = 'TITLE: \n'
    # author_quote = 'FIRST AUTHOR: \n'
    # abstract_quote = 'ABSTRACT: \n' 
    

    quote = 'TITLE\n'+ title[0] + '\n \n' + 'FIRST AUTHOR: \n' + authors + '\n \n'  + 'ABSTRACT: \n' + abstract 

    T.insert(tk.END, quote) 

    def gotolink():
        webbrowser.open(the_url,new=1)

    def searchagain():
        root.destroy()
        import main


    submit_button = tk.Button(root, text="Go To Paper", pady=0, command=gotolink,bg='white',activebackground='white',fg='black',font = ('Helvetica', 18),activeforeground='white',bd=0)
    submit_button.pack(side=tk.TOP,fill='both',expand=True)

    submit_button = tk.Button(root, text="Search Again", pady=0, command=searchagain,bg='white',activebackground='white',fg='black',font = ('Helvetica', 18),activeforeground='white',bd=0)
    submit_button.pack(side=tk.TOP,fill='both',expand=True)

    tk.mainloop()


#Test Output 
# title = ['example title goes here.'] 
# authors = 'E. Komatsu(Texas U.)'
# abstract = '(Abridged) The WMAP 5-year data strongly limit deviations from the minimal LCDM model. We constrain the physics of inflation via Gaussianity, adiabaticity, the power spectrum shape, gravitational waves, and spatial curvature. We also constrain the properties of dark energy, parity-violation, and neutrinos. We detect no convincing deviations from the minimal model. The parameters of the LCDM model, derived from WMAP combined with the distance measurements from the Type Ia supernovae (SN) and the Baryon Acoustic Oscillations (BAO), are: Omega_b=0.0462+-0.0015, Omega_c=0.233+-0.013, Omega_Lambda=0.721+-0.015, H_0=70.1+-1.3 km/s/Mpc, n_s=0.960+0.014-0.013, tau=0.084+-0.016, and sigma_8=0.817+-0.026. With WMAP+BAO+SN, we find the tensor-to-scalar ratio r<0.20 (95% CL), and n_s>1 is disfavored regardless of r. We obtain tight, simultaneous limits on the (constant) equation of state of dark energy and curvature. We provide a set of...' 
# urls =  ['http://$SIMBAD$/simbo.pl?bibcode=2009ApJS..180..330K','http://iopscience.iop.org/article/10.1088/0067-0049/180/2/330/pdf','http://inspirehep.net/search?p=find+eprint+arXiv:0803.0547']

# output(title,authors,abstract,urls)